/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.container.httpserver.HttpServerFactory;
import com.sun.jersey.api.core.PackagesResourceConfig;
import com.sun.jersey.api.core.ResourceConfig;
import com.sun.jersey.api.view.Viewable;
import com.sun.net.httpserver.HttpServer;
import cz.xelfi.quoridor.webidor.resources.Quoridor;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.Callable;
import javax.ws.rs.DefaultValue;
import javax.ws.rs.FormParam;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.NewCookie;
import javax.ws.rs.core.Response;
import org.w3c.dom.Document;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@Path("/")
public final class UI {
    private static WebResource base;

    @Context
    private HttpHeaders headers;
    private String user;

    public UI() {
    }

    private Viewable checkLogin() {
        if (headers.getCookies().containsKey("login")) {
            user = headers.getCookies().get("login").getValue();
            return null;
        }
        return new Viewable("login.fmt", null);
    }

    @POST
    @Path("login")
    @Produces(MediaType.TEXT_HTML)
    public Response login(
        @FormParam("name") String name, @FormParam("password") String password
    ) throws Exception {
        File f = new File(new File(new File(System.getProperty("user.home")), ".quoridor"), "passwd");
        Properties p = new Properties();
        try {
            p.load(new FileInputStream(f));
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        if (name != null && password.equals(p.getProperty(name))) {
            return Response.seeOther(new URI("/")).cookie(new NewCookie("login", name)).entity(welcomeImpl()).build();
        } else {
            Viewable v = new Viewable("login.fmt", "Invalid name or password: " + name);
            return Response.status(1).entity(v).build();
        }
    }

    @GET
    @Produces(MediaType.TEXT_HTML)
    public Viewable welcome() {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        return welcomeImpl();
    }

    @GET
    @Path("games/{id}/")
    @Produces(MediaType.TEXT_HTML)
    public Viewable board(@PathParam("id") String id) {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        Map<String,Object> map = new HashMap<String,Object>();
        Document doc = base.path("games").path(id).accept(MediaType.TEXT_XML).get(Document.class);
        map.put("doc", doc);
        map.put("user", user);
        return new Viewable("game.fmt", map);
    }

    @GET
    @Path("games/{id}/move")
    @Produces(MediaType.TEXT_HTML)
    public Viewable move(
        @PathParam("id") String id,
        @QueryParam("type") String type,
        @QueryParam("direction") String direction,
        @QueryParam("direction-next") @DefaultValue("") String directionNext,
        @QueryParam("column") @DefaultValue("") String column,
        @QueryParam("row") @DefaultValue("") String row
    ) {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        WebResource wr = base.path("games").path(id).queryParam("player", user);
        if (type.equals("fence")) {
            wr.queryParam("move", direction.charAt(0) + column + row).put();
            return board(id);
        }
        if (type.equals("move")) {
            wr.queryParam("move", direction + directionNext).put();
            return board(id);
        }
        return board(id);
    }

    @GET
    @Path("games/create")
    @Produces(MediaType.TEXT_HTML)
    public Viewable create(
        @QueryParam("white") String white,
        @QueryParam("black") String black
    ) {
        Viewable v = checkLogin();
        if (v != null) {
            return v;
        }
        Object obj =
            base.path("games").queryParam("white", white).
            queryParam("black", black).post(Document.class);
        return welcome();
    }

    private Viewable welcomeImpl() {
        final Object got = base.path("games").accept(MediaType.TEXT_XML).get(Document.class);
        return new Viewable("index.fmt", got);
    }

    //
    // start the server
    //

    public static void main(String[] args) throws Exception {
        Callable<Void> r = startServers();

        System.in.read();
        r.call();
        System.exit(0);
    }

    static Callable<Void> startServers() throws Exception {
        final HttpServer api = Quoridor.start(9998);
        Client client = new Client();
        base = client.resource(new URI("http://localhost:9998/api/"));

        final HttpServer s = start(9997);
        System.out.println(
            "Quoridor started at port 9997\n" + "Hit enter to stop it..."
        );

        return new Callable<Void>() {
            public Void call() throws Exception {
                s.stop(0);
                api.stop(0);
                return null;
            }
        };
    }

    static HttpServer start(int port) throws IOException {
        final String baseUri = "http://localhost:" + port + "/";
        ResourceConfig rc = new PackagesResourceConfig("cz.xelfi.quoridor.freemarkerdor");
        HttpServer server = HttpServerFactory.create(baseUri, rc);
        server.start();
        return server;
    }

}
